import { Result } from "./base";
import { Nub } from "./nub";
import { ParamsSection, acSection } from "./section/section_type";
import { ParamDefinition, ParamCalculability } from "./param"

export class RegimeUniforme extends Nub {
    private Sn: acSection;

    constructor(s: acSection, dbg: boolean = false) {
        super(s.prms, dbg);
        this.Sn = s;
    }

    protected setParametersCalculability() {
        this.prms.Q.calculability = ParamCalculability.EQUATION;
        this.prms.Y.calculability = ParamCalculability.EQUATION;
    }

    get prms(): ParamsSection {
        return <ParamsSection>this._prms;
    }

    /**
   * Calcul du débit en régime uniforme.
   * @return Débit en régime uniforme
   */
    Calc_Qn(): number {
        this.Sn.Reset(true);
        if (this.Sn.prms.If.v <= 0)
            return 0;

        // this.debug("RU: Calc_Qn : ");
        // this.debug(this.Sn.prms);

        return this.Sn.prms.Ks.v * Math.pow(this.Sn.Calc("R", this.Sn.prms.Y.v), 2 / 3) * this.Sn.Calc("S", this.Sn.prms.Y.v) * Math.sqrt(this.Sn.prms.If.v);
    }

    Equation(sVarCalc: string): Result {
        this.debug("RU: Equation(" + sVarCalc + ")")
        let v: number;

        switch (sVarCalc) {
            case 'Y':
                v = this.Sn.Calc("Yn");
                break;

            case 'Q':
                v = this.Calc_Qn();
                break;

            default:
                throw "RegimeUniforme.Equation() : invalid variable name " + sVarCalc;
        }

        return new Result(v);
    }
}